/*------------------------------------------------------------------------------
* correlator.c: mex function correlator
*
* 2010/10/15 0.1 new
*-----------------------------------------------------------------------------*/
#include "mex.h"

extern void correlator(int type, int sat, double crate, double coff,
                       double freq, double phi0, const char *buff, double ts,
                       int ns, int s, int nc, double *I, double *Q);

/* mex interface -------------------------------------------------------------*/
extern void mexFunction(int nargout, mxArray *argout[], int nargin,
                        const mxArray *argin[])
{
    double crate,coff,freq,phi0,ts,*I,*Q;
    char *data;
    int type,sat,s,ns,nc;
    
    if (nargin<10||
        mxGetM(argin[0])<1||mxGetM(argin[1])<1||mxGetM(argin[2])<1||
        mxGetM(argin[3])<1||mxGetM(argin[4])<1||mxGetM(argin[5])<1||
        mxGetM(argin[6])<1||mxGetM(argin[7])<1||mxGetM(argin[8])<1||
        mxGetM(argin[9])<1||!mxIsInt8(argin[6])) {
        mexErrMsgTxt("argin error");
    }
    type =(int)*mxGetPr(argin[0]); /* I: sampling type (0:real,1:I/Q) */
    sat  =(int)*mxGetPr(argin[1]); /* I: satellite number */
    crate=     *mxGetPr(argin[2]); /* I: code chip rate (chip/s) */
    coff =     *mxGetPr(argin[3]); /* I: code offset (chip) */
    freq =     *mxGetPr(argin[4]); /* I: frequency (Hz) */
    phi0 =     *mxGetPr(argin[5]); /* I: phase offset (rad) */
    data =      mxGetPr(argin[6]); /* I: sampling signal */
    ts   =     *mxGetPr(argin[7]); /* I: sampling interval (s) */
    s    =(int)*mxGetPr(argin[8]); /* I: correlator spacing (sample) */
    nc   =(int)*mxGetPr(argin[9]); /* I: number of side correlator */
    
    ns   =mxGetN(argin[6]);
    if (type) ns/=2;
    
    argout[0]=mxCreateDoubleMatrix(1,1+2*nc,mxREAL);
    argout[1]=mxCreateDoubleMatrix(1,1+2*nc,mxREAL);
    
    /* I=(I_P,I_E1,I_L1,I_E2,I_L2,...), Q=(Q_P,...) */
    I    =     mxGetPr(argout[0]); /* O: correlator results I */
    Q    =     mxGetPr(argout[1]); /* O: correlator results Q */
    
    correlator(type,sat,crate,coff,freq,phi0,data,ts,ns,s,nc,I,Q);
}
